@@ -291,6 +291,7 @@ void DirectoryMergeWindow::reload()
       if ( result!=KMessageBox::Yes )
          return;
    }
+
    init( m_dirA, m_dirB, m_dirC, m_dirDest, m_bDirectoryMerge );
 }
 
@@ -391,8 +392,9 @@ bool DirectoryMergeWindow::init
       checkIfCanContinue( &bCanContinue );
       if ( !bCanContinue )
          return false;
+      startDiffMerge("","","","","","","",0);  // hide main window
    }
-   
+
    show();
 
    ProgressProxy pp;
@@ -811,9 +813,19 @@ void DirectoryMergeWindow::compareFilesAndCalcAges( MergeFileInfos& mfi )
             "",
             "","","",&mfi.m_totalDiffStatus
             );
-         mfi.m_bEqualAB = mfi.m_totalDiffStatus.bBinaryAEqB;
-         mfi.m_bEqualBC = mfi.m_totalDiffStatus.bBinaryBEqC;
-         mfi.m_bEqualAC = mfi.m_totalDiffStatus.bBinaryAEqC;
+         int nofNonwhiteConflicts = mfi.m_totalDiffStatus.nofUnsolvedConflicts + 
+            mfi.m_totalDiffStatus.nofSolvedConflicts - mfi.m_totalDiffStatus.nofWhitespaceConflicts;
+
+         if (m_pOptions->m_bDmWhiteSpaceEqual && nofNonwhiteConflicts == 0)
+         {
+            mfi.m_bEqualAB =  mfi.m_bEqualBC =  mfi.m_bEqualAC = true;
+         }
+         else
+         {
+            mfi.m_bEqualAB = mfi.m_totalDiffStatus.bBinaryAEqB;
+            mfi.m_bEqualBC = mfi.m_totalDiffStatus.bBinaryBEqC;
+            mfi.m_bEqualAC = mfi.m_totalDiffStatus.bBinaryAEqC;
+         }
       }
    }
    else
@@ -2201,7 +2213,7 @@ bool DirectoryMergeWindow::renameFLD( const QString& srcName, const QString& des
       }
    }
 
-   m_pStatusInfo->addText(i18n("rename( %1 -> %2 )").arg(srcName).arg(destName))	;
+   m_pStatusInfo->addText(i18n("rename( %1 -> %2 )").arg(srcName).arg(destName));
    if ( m_bSimulatedMergeStarted )
    {
       return true;
